# (c) cavaliba.com

#  command / cavaliba.py

import os
import sys
import json
import yaml


from django.core.management.base import BaseCommand, CommandError
from django.forms.models import model_to_dict


from app_user.aaa import start_command
from app_user.aaa import get_all_perms

#from app_home.home import cavaliba_init
from app_home.home import cavaliba_update


# version
from app_home.cavaliba import CAVALIBA_VERSION

# log
from app_home.log import log_get_count
from app_home.log import log_list
from app_home.log import purge
from app_home.log import purge_all

# conf
from app_home.configuration import conf_list

# load
from app_data.loader import load_file_yaml
from app_data.loader import load_file_json
from app_data.loader import load_file_csv
from app_data.pipeline import apply_pipeline
from app_data.pipeline import get_pipeline
from app_data.loader import load_broker


# export
from app_user.models import SireneUser
from app_user.models import SireneGroup
from app_user.models import SirenePermission
from app_data.models import DataClass
from app_user.user import user_listdict_format
from app_user.group import group_listdict_format
from app_user.role import role_listdict_format
from app_user.permission import permission_listdict_format
from app_sirene.export import sirene_export_dict
from app_data.exporter import cli_listdict_format
from app_data.exporter import cli_class_listdict_format
from app_home.export import home_export_dict

# export
SCHEMA_LIST = ["user","group","role","permission", "home",
               "sirene", "sirene_category", "sirene_template", "sirene_public",
               "apikey", "dataview","enumerate","pipeline",
               "dataclass"
               ]

# YAML export
class MyYamlDumper(yaml.SafeDumper):
    def write_line_break(self, data=None):
        super().write_line_break(data)
        if len(self.indents) < 2:
            super().write_line_break()



class Command(BaseCommand):

    help = 'manage cavaliba configuration'

    # https://docs.python.org/3/library/argparse.html#name-or-flags
    def add_arguments(self, parser):

        parser.add_argument('--verbose', action='store_true', help='Verbose mode' )

        parser.add_argument('action', type=str,  help="version|update|log|conf|load|export" )      

        # log
        parser.add_argument('--log_purge_all', action='store_true', help='log: purge ALL log entries' )
        parser.add_argument('--log_purge', action='store_true', help='log: purge OLDER log entries' )
        parser.add_argument('--log_list', action='store_true', default=True, help="log: list recent logs(default)")
        parser.add_argument('--log_start', type=int, default=0, help="log: start position")
        parser.add_argument('--log_end', type=int, default=10, help="log: end position")

        # conf
        #parser.add_argument('--conf_list', action='store_true', help="conf: list conf entries")

        # load
        parser.add_argument('--pipeline', type=str, help="load: pipeline to preprocess data")
        parser.add_argument('--force', type=str, help="load: override _action field in files")      
        parser.add_argument('--first', type=int, default=1, help="load: first line to load. default 1")
        parser.add_argument('--last', type=int, default=0, help="load: last line to load. Default 0 (last)")
        parser.add_argument('--pass', type=int, default=1, help="load: default 1 , increase number for relationships between objects")
        parser.add_argument('--dryrun', action='store_true', help='load: Dry run, no loading. Use verbose to see data' )
        parser.add_argument('--progress', action='store_true', help='load: Display progress' )
        # parser.add_argument('--test', action='store_true', help='Test/do nothing' )
        parser.add_argument('filenames', nargs='*', type=str)


        # export
        parser.add_argument('--list', action='store_true', help="export: list available schemas")
        parser.add_argument('--schema', type=str, help="export: see available values")
        parser.add_argument('--format', type=str, help="export: yaml, json, csv, text")
        parser.add_argument('--key', type=str, help="export: select specific keyname")



    def handle(self, *args, **options):

        action = options.get("action", None)
        verbose = options.get("verbose", False)

        valid_action = [ "update", "log", "conf", "load", "export", "version"]
        if action:
            if action not in valid_action:
                self.stderr.write("unknown action: " + action)
                self.stderr.write("valid action: " +  ' '.join(valid_action) )
                sys.exit(1)

        start_command(command="cavaliba", action=action)

        # # INIT
        # if action == "init":
        #     cavaliba_init()
        #     self.stdout.write("init done.\n")

        # UPDATE
        if action == "update":
            cavaliba_update(verbose=verbose)
            self.stdout.write("update done.\n")

        # LOAD
        if action == "load":
            self.cavaliba_load(options)

        # LOG
        elif action == "log":

            self.cavaliba_log(options)

        # LOG
        elif action == "conf":
            self.cavaliba_conf(options)

        # EXPORT
        elif action == "export":
            self.cavaliba_export(options)

        # VERSION
        elif action == "version":
            print(CAVALIBA_VERSION)


    # ----------------------------------------------------------------
    # conf
    # ----------------------------------------------------------------

    def cavaliba_conf(self, options):
        
        key = options.get("key", None)
        format = options.get("format")
        if not format:
            format = "text"
        if format not in ["yaml", "json","text","csv"]:
            self.stderr.write(f"Unknown format: {format}")
            exit(0)

        dict_attributs =  [ "appname", "keyname", "value", "description", "order", "page"]
        conf_objects = conf_list()
        datalist = []
        for entry in conf_objects:

            m = model_to_dict(entry, fields=dict_attributs)

            if key:
                if m['keyname'] != key:
                    continue

            datalist.append(m)

        if format == "yaml":
            filedata = yaml.dump(datalist, allow_unicode=True, Dumper=MyYamlDumper, sort_keys=False)
            self.stdout.write(f"{filedata}")
        elif format == "json":
            filedata = json.dumps(datalist, indent=4, ensure_ascii = False)
            self.stdout.write(f"{filedata}")
        elif format == "csv":
            pass
        else:
            for i in datalist:
                appname = i['appname']
                keyname = i['keyname']
                value = i['value']
                self.stdout.write(f"{appname:10} {keyname:40} {value}" )
            self.stdout.write("conf done\n")



    def cavaliba_conf2(self, options):
  
        conf_objects = conf_list()

        for i  in conf_objects:
            self.stdout.write(f"{i.appname:10} {i.keyname:40} {i.value}" )
 
        self.stdout.write("conf done\n")


    # ----------------------------------------------------------------
    # log
    # ----------------------------------------------------------------

    def cavaliba_log(self, options):

            want_list = options.get("log_list")
            want_purge = options.get("log_purge",False)
            want_purge_all = options.get("log_purge_all",False)
          
            log_start = options.get("log_start",0)
            log_end = options.get("log_end",10)

            count = 0

            count = log_get_count()
            self.stdout.write(f"log entries: {count}\n")


            if want_list:
                r = log_list(log_start,log_end)
                for i in r:
                    self.stdout.write(f"{i}")

            elif want_purge:
                count = purge()
                self.stdout.write(f"log deleted (old) : {count} entries")

            elif want_purge_all:
                count = purge_all()
                self.stdout.write(f"log deleted (all) : {count} entries")

            else:
                pass
                
        
            self.stdout.write("log done\n")




    # ----------------------------------------------------------------
    # load
    # ----------------------------------------------------------------
    # load data from CSV/JSON/YAML file to DB

    def cavaliba_load(self, options):

        verbose = options.get("verbose", True)

        self.stdout.write("="*60)
        first = options.get("first", 1)
        last = options.get("last", 0)
        if last > 0 and last < first:
            self.stderr.write("ERR - invalid first/last values")
            exit(0)
        self.stdout.write(f"first/last:  {first}, {last}")


        # pipeline
        pipeline = options.get("pipeline", None)
        # TODO : chech w/ regexp
        if pipeline:
            self.stdout.write(f"pipeline:    {pipeline}")
            pipeline_data = get_pipeline(pipeline)
            if not pipeline_data:
                self.stderr.write(f"ERR - invalid pipeline {pipeline}")
                exit(0)
            if verbose:
                self.stdout.write(json.dumps(pipeline_data, indent=2, ensure_ascii=False))

        # force_action
        force_action = options.get("force", None)
        if force_action:
            if force_action not in ["create", "init","update", "delete", "enable", "disable"]:
                self.stderr.write(f"ERR - invalid force_action {force_action}")
                exit(0)
            self.stdout.write(f"force_action: {force_action}")

        # pass count
        passcount = options.get("pass", 1)
        self.stdout.write(f"pass:        {passcount}")

        # dryrun
        dryrun = options.get("dryrun", None)
        self.stdout.write(f"dryrun:      {dryrun}")

        # progress
        progress = options.get("progress", False)
        self.stdout.write(f"progress:    {progress}")

        # build file list ordered
        ordered = []
        for fileentry in options['filenames']:
            if os.path.isfile(fileentry):
                ordered.append(fileentry)
            elif os.path.isdir(fileentry):
                for item in os.scandir(fileentry):
                    if item.is_file(follow_symlinks=False):
                        if item.path not in ordered:
                            ordered.append(item.path)
                ordered.sort()
            else:
                self.stderr.write(f"SKIP - Unknown filename: {fileentry}")
                

        for passcurrent in range(1,passcount+1):

            self.stdout.write("="*60)
            self.stdout.write(f"pass : {passcurrent} / {passcount}")
            self.stdout.write("="*60)


            for filename in ordered:

                self.stdout.write()
                self.stdout.write(f"File: {filename} ...")
                datalist = []

                if filename.endswith('.csv'):
                    datalist = load_file_csv(filename,pipeline=pipeline,first=first,last=last)

                elif filename.endswith('.yml') or filename.endswith('.yaml'):
                    datalist = load_file_yaml(filename,pipeline=pipeline,verbose=verbose)

                elif filename.endswith('.json'):
                    datalist = load_file_json(filename,pipeline=pipeline,verbose=verbose)

                else:
                    self.stderr.write(f"SKIP - Unknown filtetype ({filename})")
                    continue

                if not datalist:
                    self.stderr.write(f"SKIP - no data in ({filename})")
                    continue
                    
                if len(datalist) == 0:
                    self.stderr.write(f"SKIP - empty data in ({filename})")
                    continue

                self.stdout.write(f"Found: {len(datalist)} objects")

                if verbose:
                    self.stdout.write(json.dumps(datalist, indent=2, ensure_ascii=False))
                
                # apply pipeline
                print(f"Pipeline: {pipeline}")
                if pipeline:
                    datalist = apply_pipeline(pipeline=pipeline, datalist=datalist)

                if verbose:
                    self.stdout.write(json.dumps(datalist, indent=2, ensure_ascii=False))

                # load to DB
                if not dryrun:
                    self.stdout.write("Loading to DB...")
                    aaa = {}
                    aaa["perms"] = get_all_perms()

                    r = load_broker(datalist, aaa=aaa, force_action=force_action, verbose=verbose, progress=progress)
                    #if verbose:
                    self.stdout.write(f"Loaded: {r} objects")
                else:
                    self.stdout.write("Dryrun : no DB action")

        # END
        self.stdout.write("load done\n")



    # ----------------------------------------------------------------
    # export
    # ----------------------------------------------------------------

    def cavaliba_export(self, options):


        #verbose = options.get("verbose", True)

        schema = options.get("schema", None)
        format = options.get("format")
        if not format:
            format = "yaml"
        if format not in ["yaml", "json", "text", "csv"]:
            self.stderr.write(f"Unknown format: {format}")
            exit(0)

        key = options.get("key", None)

        # --list
        wantlist = options.get("list", None)
        if wantlist:
            self.stdout.write("Built-In schemas:")
            for i in SCHEMA_LIST:
                self.stdout.write(f"- {i}")
            self.stdout.write()
            
            print("Custom schemas:")
            classes = DataClass.objects.all()
            for i in classes:
                if i.keyname in SCHEMA_LIST:
                    continue
                if i.keyname.startswith('_'):
                    if i.keyname[1:] in SCHEMA_LIST:
                        continue
                self.stdout.write(f"- {i}")

            exit(0)

        filedata= []
        datalist = []

        # user
        if schema == "user":
            if key:
                users = SireneUser.objects.filter(login=key)
            else:
                users = SireneUser.objects.order_by('login').all()
            datalist = user_listdict_format(users)

        # group
        elif schema == "group":
            if key:
                groups = SireneGroup.objects\
                    .filter(is_role=False, keyname=key)\
                    .prefetch_related("users")\
                    .prefetch_related("subgroups")\
                    .order_by('keyname')
            else:
                groups = SireneGroup.objects\
                    .filter(is_role=False)\
                    .prefetch_related("users")\
                    .prefetch_related("subgroups")\
                    .order_by('keyname')
            datalist = group_listdict_format(groups)

        # role
        elif schema == "role":
            if key:
                roles = SireneGroup.objects\
                    .filter(is_role=True, keyname=key)\
                    .prefetch_related("permissions")\
                    .prefetch_related("users")\
                    .prefetch_related("subgroups")\
                    .order_by('keyname')

            else:
                roles = SireneGroup.objects\
                    .filter(is_role=True)\
                    .prefetch_related("permissions")\
                    .prefetch_related("users")\
                    .prefetch_related("subgroups")\
                    .order_by('keyname')
            datalist = role_listdict_format(roles)

        # permission
        elif schema == "permission":
            if key:
                permissions = SirenePermission.objects.filter(keyname=key)
            else:
                permissions = SirenePermission.objects.all()
            datalist = permission_listdict_format(permissions)

        # sirene
        elif schema == "sirene":
            datalist = sirene_export_dict()

        elif schema == "sirene_public":
            datalist = sirene_export_dict(category=False, template=False, public=True)

        elif schema == "sirene_template":
            datalist = sirene_export_dict(category=False, template=True, public=False)

        elif schema == "sirene_category":
            datalist = sirene_export_dict(category=True, template=False, public=False)

        # home
        elif schema == "home":
            datalist = home_export_dict(keyname=key)
            

        # _builtin(s)
        elif schema in ["apikey", "enumerate", "dataview", "pipeline"]:
            classes = ["_" + schema]
            if key:
                datalist = cli_listdict_format(classes=classes, keyname=key)
            else:
                datalist = cli_listdict_format(classes=classes)            
            datalist = cli_listdict_format(classes)

        elif schema == "dataclass":
            if key:
                datalist = cli_class_listdict_format(classes=[key])
            else:
                datalist = cli_class_listdict_format()

        # schema
        else:
            classes = [schema]
            if key:
                datalist = cli_listdict_format(classes=classes, keyname=key)
            else:
                datalist = cli_listdict_format(classes=classes)

        # ----

        if format == "yaml":
            filedata = yaml.dump(datalist, allow_unicode=True, Dumper=MyYamlDumper, sort_keys=False)

        elif format == "json":
            filedata = json.dumps(datalist, indent=4, ensure_ascii = False)

        elif format == "csv":
            pass

        else:
            exit(0)
            
        self.stdout.write(f"{filedata}")
