# (c) cavaliba.com - test_data 

import yaml 

from django.test import TestCase, override_settings
from django.test import TransactionTestCase
from django.urls import reverse

from app_home.home import cavaliba_update
from app_home.configuration import get_configuration

import app_home.cache as cache

from app_data.models import DataClass
from app_data.models import DataSchema
from app_data.models import DataInstance

from app_data.data import Instance
from app_data.loader import load_broker

from app_data.fieldtypes.field_ipv4 import FieldIPV4
from app_data.fieldtypes.field_float import FieldFloat
from app_data.fieldtypes.field_int import FieldInt




class TestDataClass(TestCase):
    
    fixtures = ["test"]

    def setUp(self):
        cache.init()
        

    def test_has_apikey(self):
        classobj = DataClass.objects.get(keyname='_apikey')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)      

    def test_has_enumerate(self):
        classobj = DataClass.objects.get(keyname='_enumerate')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)      

    def test_has_pipeline(self):
        classobj = DataClass.objects.get(keyname='_pipeline')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)      

    def test_has_dataview(self):
        classobj = DataClass.objects.get(keyname='_dataview')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)      


    def test_has_site(self):
        classobj = DataClass.objects.get(keyname='site')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)


    def test_has_app(self):
        classobj = DataClass.objects.get(keyname='app')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)

    def test_has_sitegroup(self):
        classobj = DataClass.objects.get(keyname='sitegroup')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)

    def test_has_customer(self):
        classobj = DataClass.objects.get(keyname='customer')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)



    def test_has_schema_test1(self):
        classobj = DataClass.objects.get(keyname='test1')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)

    def test_has_schema_test2(self):
        classobj = DataClass.objects.get(keyname='test2')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)

    def test_has_schema_test3(self):
        classobj = DataClass.objects.get(keyname='test3')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)

    def test_has_schema_test4(self):
        classobj = DataClass.objects.get(keyname='test4')
        self.assertIsNotNone(classobj)
        self.assertEqual(classobj.is_enabled, True)


    def test_schema_count(self):

        DEFAULT = {
            'app':2,
            'site':3,
            'customer':2,
            'sitegroup':4,
            'test1':27,
            'test2':1,
            'test3':2,
            'test4':2,
            '_apikey':12,
            '_enumerate':2,
            '_dataview': 3,
            '_pipeline':2,

        }

        for classname,schemacount in DEFAULT.items():
            classobj = DataClass.objects.get(keyname=classname)
            count = DataSchema.objects.filter(classobj=classobj).count()
            self.assertEqual(count, schemacount)




    def test_schema_delete_allowed(self):
        datalist = yaml.safe_load('''
            - classname: _schema
              keyname: test1
              _action: delete
            ''')
        aaa = {'perms':['p_schema_delete']}
        r = load_broker(datalist=datalist, aaa=aaa, verbose=False)

        classobj = DataClass.objects.filter(keyname='test1').first()
        self.assertIsNone(classobj)

    def test_schema_delete_not_allowed(self):
        datalist = yaml.safe_load('''
            - classname: _schema
              keyname: test1
              _action: delete
            ''')
        aaa = {'perms':['p_unknown_permission']}
        r = load_broker(datalist=datalist, aaa=aaa, verbose=False)

        classobj = DataClass.objects.filter(keyname='test1').first()
        self.assertIsNotNone(classobj)

# --

    def test_schema_disable_enable_allowed(self):
        datalist = yaml.safe_load('''
            - classname: _schema
              keyname: test1
              _action: disable
            ''')
        aaa = {'perms':['p_schema_update']}

        load_broker(datalist=datalist, aaa=aaa, verbose=False)
        classobj = DataClass.objects.filter(keyname='test1').first()
        self.assertEqual(classobj.is_enabled, False)

        datalist = yaml.safe_load('''
            - classname: _schema
              keyname: test1
              _action: enable
            ''')
        load_broker(datalist=datalist, aaa=aaa, verbose=False)
        classobj = DataClass.objects.filter(keyname='test1').first()
        self.assertEqual(classobj.is_enabled, True)


    def test_schema_disable_enable_not_allowed(self):
        datalist = yaml.safe_load('''
            - classname: _schema
              keyname: test1
              _action: disable
            ''')
        aaa = {'perms':['p_schema_read']}
        load_broker(datalist=datalist, aaa=aaa, verbose=False)

        classobj = DataClass.objects.filter(keyname='test1').first()
        self.assertEqual(classobj.is_enabled, True)

# schema field

    def test_schemafield_delete_allowed(self):
        datalist = yaml.safe_load('''
            - classname: _schema
              keyname: test0
              mystring:
                  _action: delete
            ''')
        aaa = {'perms':['p_schemafield_delete']}
        r = load_broker(datalist=datalist, aaa=aaa, verbose=False)

        obj = DataSchema.objects.filter(classname='test0', keyname='mystring').first()
        self.assertIsNone(obj)

        instance = Instance(classname="test0")  
        self.assertNotIn('mystring', instance.fields)


    def test_schemafield_delete_not_allowed(self):
        datalist = yaml.safe_load('''
            - classname: _schema
              keyname: test0
              mystring:
                  _action: delete
            ''')
        aaa = {'perms':['p_unknown_permission']}
        r = load_broker(datalist=datalist, aaa=aaa, verbose=False)

        instance = Instance(classname="test0", iname="test0-01")
        self.assertIn('mystring', instance.fields)


# Instances

    # export/import

    # read

    def test_instance_read_allowed_class(self):
        instance = Instance(classname="test1", iname="test1-01")
        aaa = {'perms':['p_test1_read']}
        r = instance.has_read_permission(aaa=aaa)
        self.assertTrue(r)

    def test_instance_read_allowed_allclass(self):
        instance = Instance(classname="test0", iname="test0-01")
        #instance.print()
        self.assertEqual(instance.displayname,"test0-01")
        aaa = {'perms':['p_data_read']}
        r = instance.has_read_permission(aaa=aaa)
        self.assertTrue(r)

    def test_instance_read_allowed_admin(self):
        instance = Instance(classname="test1", iname="test1-01")
        aaa = {'perms':['p_data_admin']}
        r = instance.has_read_permission(aaa=aaa)
        self.assertTrue(r)


    def test_instance_read_not_allowed(self):
        instance = Instance(classname="test1", iname="test1-01")
        aaa = {'perms':['p_test1_read2']}
        r = instance.has_read_permission(aaa=aaa)
        self.assertFalse(r)


    # delete

    def test_instance_delete_allowed(self):
        datalist = yaml.safe_load('''
            - classname: test1
              keyname: 'test1-01'
              _action: delete
            ''')
        aaa = {'perms':['p_test1_delete']}
        load_broker(datalist=datalist, aaa=aaa, verbose=True)

        # removed from DB
        classobj = DataClass.objects.filter(keyname='test1').first()
        iobj = DataInstance.objects.filter(classobj=classobj, keyname="test1-01").first()
        self.assertIsNone(iobj)

        # removed from Instance() cache
        instance = Instance(classname="test1", iname="test1-01")
        #instance.print()
        self.assertFalse(instance.is_bound())


    def test_instance_delete_not_allowed(self):
        datalist = yaml.safe_load('''
            - classname: test1
              keyname: 'test1-01'
              _action: delete
            ''')
        aaa = {'perms':['p_test1_delete2']}
        load_broker(datalist=datalist, aaa=aaa, verbose=True)

        # not removed from DB
        classobj = DataClass.objects.filter(keyname='test1').first()
        iobj = DataInstance.objects.filter(classobj=classobj, keyname="test1-01").first()
        self.assertIsNotNone(iobj)

        # not removed from Instance() cache
        instance = Instance(classname="test1", iname="test1-01")
        self.assertTrue(instance.is_bound())


    # enable / diable

    def test_instance_enable_allowed(self):

        datalist = yaml.safe_load('''
            - classname: test1
              keyname: 'test1-01'
              _action: disable
            ''')
        aaa = {'perms':['p_test1_update']}
        load_broker(datalist=datalist, aaa=aaa, verbose=True)
        instance = Instance(classname="test1", iname="test1-01")
        self.assertFalse(instance.is_enabled)

        datalist = yaml.safe_load('''
            - classname: test1
              keyname: 'test1-01'
              _action: enable
            ''')
        
        aaa = {'perms':['p_test1_update']}
        load_broker(datalist=datalist, aaa=aaa, verbose=True)
        instance = Instance(classname="test1", iname="test1-01")
        self.assertTrue(instance.is_enabled)


    def test_instance_enable_not_allowed(self):
        datalist = yaml.safe_load('''
            - classname: test1
              keyname: 'test1-01'
              _action: disable
            ''')
        aaa = {'perms':['p_test1_update']}
        load_broker(datalist=datalist, aaa=aaa, verbose=True)
        instance = Instance(classname="test1", iname="test1-01")
        self.assertFalse(instance.is_enabled)

        datalist = yaml.safe_load('''
            - classname: test1
              keyname: 'test1-01'
              _action: enable
            ''')
        aaa = {'perms':['p_test1_update2']}
        load_broker(datalist=datalist, aaa=aaa, verbose=True)
        instance = Instance(classname="test1", iname="test1-01")
        self.assertFalse(instance.is_enabled)



    def test_instance_disable_not_allowed(self):
        datalist = yaml.safe_load('''
            - classname: test1
              keyname: 'test1-01'
              _action: disable
            ''')
        aaa = {'perms':['p_test1_update2']}
        load_broker(datalist=datalist, aaa=aaa, verbose=True)

        instance = Instance(classname="test1", iname="test1-01")
        #instance.print()
        self.assertTrue(instance.is_enabled)



# Fields

    def test_field_values(self):

        instance = Instance(classname="test1", iname="test1-01", expand=False)
        self.assertIsNotNone(instance)
        self.assertEqual(instance.fields["mystring"].get_value(),["data for test1-01"])
        self.assertEqual(instance.fields["myint"].get_value(),[11])
        self.assertEqual(instance.fields["myfloat"].get_value(),[27.19])
        self.assertEqual(instance.fields["myboolean"].get_value(),[True])
        self.assertEqual(instance.fields["mydate"].get_value(),["2024-11-01"])
        self.assertEqual(instance.fields["myipv4"].get_value(),["10.1.2.3"])
        self.assertEqual(instance.fields["my_schema_test0"].get_value(),["test0-01"])
        self.assertEqual(instance.fields["my_schema_test0_multi"].get_value(),["test0-02","test0-03"])
        self.assertEqual(instance.fields["my_schema_self"].get_value(),["test1-03"])

    def test_default_value(self):
        instance = Instance(classname="test1")
        self.assertEqual(instance.fields["mystring_default"].get_value(), ["hello"] )
        self.assertEqual(instance.fields["myint_default"].get_value(), [11] )
        self.assertEqual(instance.fields["myint_default_multi"].get_value(), [11,22] )
        self.assertEqual(instance.fields["myfloat_default"].get_value(), [11.11] )
        self.assertEqual(instance.fields["myfloat_default_multi"].get_value(), [11.11,22.22] )


    def test_external_no_expand(self):

        instance = Instance(classname="test4", iname="test4-01", expand=False)
        self.assertIsNotNone(instance)
        self.assertIsNotNone(instance.fields["_test1"])
        self.assertFalse("test1" in instance.fields)


    def test_external_expand(self):

        instance = Instance(classname="test4", iname="test4-01", expand=True)
        self.assertIsNotNone(instance)
        self.assertIsNotNone(instance.fields["_test1"])
        self.assertIsNotNone(instance.fields["test1"])
        self.assertIsNotNone(instance.fields["test1__mystring"])
        self.assertIsNotNone(instance.fields["test1__myint"])
        self.assertIsNotNone(instance.fields["test1__myfloat"])
        self.assertIsNotNone(instance.fields["test1__myboolean"])
        self.assertEqual(instance.fields["test1__myint"].get_value(),[11])
        self.assertEqual(instance.fields["test1__myfloat"].get_value(),[27.19])
        self.assertEqual(instance.fields["test1__myboolean"].get_value(),[True])
        self.assertEqual(instance.fields["test1__mystring"].get_value(),["data for test1-01"])


    def test_enumerate_no_expand(self):

        instance = Instance(classname="test1", iname="test1-01", expand=False)
        self.assertIsNotNone(instance)
        self.assertIsNotNone(instance.fields["my_enumerate_abcde"])
        self.assertEqual(instance.fields["my_enumerate_abcde"].get_value(),["A"])
        self.assertIsNotNone(instance.fields["my_enumerate_abcde_multi"])
        self.assertTrue("A" in instance.fields["my_enumerate_abcde_multi"].get_value())
        self.assertTrue("B" in instance.fields["my_enumerate_abcde_multi"].get_value())
        self.assertTrue("C" in instance.fields["my_enumerate_abcde_multi"].get_value())

        self.assertIsNotNone(instance.fields["my_enumerate_subfields"])
        self.assertEqual(instance.fields["my_enumerate_subfields"].get_value(),["A"])
        self.assertFalse("my_enumerate_subfields__widget" in instance.fields)
        self.assertFalse("my_enumerate_subfields__enumstring" in instance.fields)
        self.assertFalse("my_enumerate_subfields__enumint" in instance.fields)
        self.assertFalse("my_enumerate_subfields__enumfloat" in instance.fields)
        self.assertFalse("my_enumerate_subfields__enumboolean" in instance.fields)

    def test_enumerate_expand(self):

        instance = Instance(classname="test1", iname="test1-01", expand=True)
        self.assertIsNotNone(instance)
        self.assertIsNotNone(instance.fields["my_enumerate_abcde"])
        self.assertEqual(instance.fields["my_enumerate_abcde"].get_value(),["A"])
        self.assertIsNotNone(instance.fields["my_enumerate_abcde_multi"])
        self.assertTrue("A" in instance.fields["my_enumerate_abcde_multi"].get_value())
        self.assertTrue("B" in instance.fields["my_enumerate_abcde_multi"].get_value())
        self.assertTrue("C" in instance.fields["my_enumerate_abcde_multi"].get_value())

        self.assertIsNotNone(instance.fields["my_enumerate_subfields"])
        self.assertEqual(instance.fields["my_enumerate_subfields"].get_value(),["A"])
        self.assertTrue("my_enumerate_subfields__widget" in instance.fields)
        self.assertTrue("my_enumerate_subfields__enumstring" in instance.fields)
        self.assertTrue("my_enumerate_subfields__enumint" in instance.fields)
        self.assertTrue("my_enumerate_subfields__enumfloat" in instance.fields)
        self.assertTrue("my_enumerate_subfields__enumboolean" in instance.fields)

        self.assertEqual(instance.fields["my_enumerate_subfields__enumint"].get_value(), [111])
        self.assertEqual(instance.fields["my_enumerate_subfields__enumfloat"].get_value(), [111.1])
        self.assertEqual(instance.fields["my_enumerate_subfields__enumstring"].get_value(), ["AAA"])
        self.assertEqual(instance.fields["my_enumerate_subfields__enumboolean"].get_value(), [False])
        self.assertEqual(instance.fields["my_enumerate_subfields__widget"].get_value(), ["&#x1F7E2;"])


# my_enumerate_subfields__enumstring
#     from: my_enumerate_subfields : A
# - Widget:
#     from: my_enumerate_subfields__widget : green_circle
# - String:
#     from: my_enumerate_subfields__enumstring   : AAA
# - Boolean:
#     from: my_enumerate_subfields__enumboolean : False
# - Int:
#     from: my_enumerate_subfields__enumint : 111
# - Float:
#     from: my_enumerate_subfields__enumfloat : 111.1



    def test_int_valid(self):

        fieldschema = {'cardinal_min':0, 'cardinal_max':1, 'default_value':''}

        # OK
        f = FieldInt( fieldname='data', fieldschema=fieldschema, alljson={'data':[33]} )
        self.assertEqual( f.is_valid(), True)
        f = FieldInt( fieldname='data', fieldschema=fieldschema, alljson={'data':[-3]} )
        self.assertEqual( f.is_valid(), True)

        # KO
        f = FieldInt( fieldname='data', fieldschema=fieldschema, alljson={'data':["a"]} )
        self.assertEqual( f.is_valid(), False)
        f = FieldInt( fieldname='data', fieldschema=fieldschema, alljson={'data':[33.4]} )
        self.assertEqual( f.is_valid(), False)


    def test_float_valid(self):

        fieldschema = {'cardinal_min':0, 'cardinal_max':1, 'default_value':''}

        # OK
        f = FieldFloat( fieldname='data', fieldschema=fieldschema, alljson={'data':[33.2]} )
        self.assertEqual( f.is_valid(), True)

        # KO
        f = FieldFloat( fieldname='data', fieldschema=fieldschema, alljson={'data':["a"]} )
        self.assertEqual( f.is_valid(), False)


    def test_ipv4_valid(self):

        fieldschema = {'cardinal_min':0, 'cardinal_max':1, 'default_value':''}

        # OK
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['10.1.2.3/24']} )
        self.assertEqual( f.is_valid(), True)
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['10.1.2.3']} )
        self.assertEqual( f.is_valid(), True)
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['0.0.0.0']} )
        self.assertEqual( f.is_valid(), True)
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['255.255.255.255']} )
        self.assertEqual( f.is_valid(), True)


        # KO
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['10.1.2.256']} )
        self.assertEqual( f.is_valid(), False)
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['10.1.2']} )
        self.assertEqual( f.is_valid(), False)
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['10.1.2.3.4']} )
        self.assertEqual( f.is_valid(), False)
        f = FieldIPV4( fieldname='myip', fieldschema=fieldschema, alljson={'myip':['10.1.2.3/33']} )
        self.assertEqual( f.is_valid(), False)
        








